package Fractal2D;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.EventQueue;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;

import javax.swing.WindowConstants;
import javax.swing.SwingUtilities;

import com.jme.input.KeyInput;
import com.jme.system.DisplaySystem;
import com.jme.system.canvas.JMECanvas;
import com.jme.system.lwjgl.LWJGLSystemProvider;
import com.jmex.awt.input.AWTMouseInput;
import com.jmex.awt.lwjgl.LWJGLAWTCanvasConstructor;

public class Fractal2D extends javax.swing.JFrame {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Canvas canvas1;
	private FractalConfigJPanel fractalConfigJPanel1;
	private JMenuItem jMenuItem1;
	private JSeparator jSeparator1;
	private ButtonGroup buttonGroup1;
	private JRadioButtonMenuItem jRadioButtonMenuItem2;
	private JRadioButtonMenuItem jRadioButtonMenuItem1;
	private JMenu jMenu1;
	private JMenuBar jMenuBar1;
	private FractalImplementor oFractalImplementor;
	private boolean fractalSwitch = true;
	private DisplaySystem display;

	// private Thread implThread;

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Fractal2D inst = new Fractal2D();
				inst.setLocationRelativeTo(null);
				inst.setVisible(true);
			}
		});
	}

	public Fractal2D() {
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
		initGUI();
	}

	private void initGUI() {
		try {
			setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			this.setTitle("Fractal2D - Speed mode");
			getContentPane().setBackground(new java.awt.Color(255, 255, 255));
			this.setIconImage(new ImageIcon(getClass().getClassLoader()
					.getResource("Fractal2D/stern_icon.png")).getImage());
			{
				jMenuBar1 = new JMenuBar();
				setJMenuBar(jMenuBar1);
				{
					jMenu1 = new JMenu();
					jMenuBar1.add(jMenu1);
					jMenu1.setText("Fractal");
					jMenu1.setMnemonic(java.awt.event.KeyEvent.VK_F);
					{
						jRadioButtonMenuItem1 = new JRadioButtonMenuItem();
						jMenu1.add(jRadioButtonMenuItem1);
						jRadioButtonMenuItem1.setText("Mandelbrot");
						jRadioButtonMenuItem1
								.setMnemonic(java.awt.event.KeyEvent.VK_M);
						jRadioButtonMenuItem1.setSelected(true);
						jRadioButtonMenuItem1
								.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent evt) {
										if (!fractalSwitch) {
											fractalConfigJPanel1
													.setMandelbrot(true);
											fractalSwitch = true;
										}
									}
								});
						getButtonGroup1().add(jRadioButtonMenuItem1);
					}
					{
						jRadioButtonMenuItem2 = new JRadioButtonMenuItem();
						jMenu1.add(jRadioButtonMenuItem2);
						jMenu1.add(getJSeparator1());
						jMenu1.add(getJMenuItem1());
						jRadioButtonMenuItem2.setText("Julia");
						jRadioButtonMenuItem2
								.setMnemonic(java.awt.event.KeyEvent.VK_J);
						jRadioButtonMenuItem2
								.addActionListener(new ActionListener() {
									public void actionPerformed(ActionEvent evt) {
										if (fractalSwitch) {
											fractalConfigJPanel1
													.setMandelbrot(false);
											fractalSwitch = false;
										}
									}
								});
						getButtonGroup1().add(jRadioButtonMenuItem2);
					}
				}
			}

			{
				fractalConfigJPanel1 = new FractalConfigJPanel();
				getContentPane().add(fractalConfigJPanel1, BorderLayout.NORTH);
				fractalConfigJPanel1.setBackground(new java.awt.Color(255, 255,
						255));
			}
			{
				KeyInput.setProvider(KeyInput.INPUT_AWT);
				display = DisplaySystem
						.getDisplaySystem(LWJGLSystemProvider.LWJGL_SYSTEM_IDENTIFIER);
				Logger.getLogger("com.jme").setLevel(Level.FINEST);
				Logger.getLogger("com.jmex").setLevel(Level.FINEST);
				display.registerCanvasConstructor("AWT",
						LWJGLAWTCanvasConstructor.class);
				canvas1 = (Canvas) display.createCanvas(640, 480);

				AWTMouseInput.setup(canvas1, false);
				canvas1.addComponentListener(new ComponentAdapter() {
					public void componentResized(ComponentEvent ce) {
						doResize();
					}
				});
				oFractalImplementor = new FractalImplementor(640, 480,
						fractalConfigJPanel1, this);
				this.fractalConfigJPanel1
						.setFractalImplmentor(oFractalImplementor);
				JMECanvas jmeCanvas = ((JMECanvas) canvas1);
				jmeCanvas.setImplementor(oFractalImplementor);
				jmeCanvas.setUpdateInput(true);
				// canvas1.setUpdateInput(true);
				canvas1.setBounds(0, 0, 640, 480);
				// canvas1.setMinimumSize(new java.awt.Dimension(640,480));
				getContentPane().add(canvas1, BorderLayout.CENTER);

			}

			pack();

			Toolkit.getDefaultToolkit().getSystemEventQueue().push(
					new EventQueue() {
						protected void dispatchEvent(AWTEvent event) {
							if (event instanceof KeyEvent) {
								KeyEvent keyEvent = (KeyEvent) event;
								if (keyEvent.getID() == KeyEvent.KEY_TYPED) {
									if (keyEvent.getKeyChar() == KeyEvent.VK_SPACE) {
										fractalConfigJPanel1.setBack();
									}

								}
							}
							super.dispatchEvent(event);

						}
					});
			this.setMinimumSize(this.getSize());
			canvas1.requestFocus();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private ButtonGroup getButtonGroup1() {
		if (buttonGroup1 == null) {
			buttonGroup1 = new ButtonGroup();
		}
		return buttonGroup1;
	}

	// Overridden so we can exit when window is closed
	protected void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			System.exit(0);
		}
	}

	protected void doResize() {
		oFractalImplementor.resizeCanvas(canvas1.getWidth(), canvas1
				.getHeight());
		this.fractalConfigJPanel1.setFractalImplmentor(oFractalImplementor);
	}

	private JSeparator getJSeparator1() {
		if (jSeparator1 == null) {
			jSeparator1 = new JSeparator();
		}
		return jSeparator1;
	}

	private JMenuItem getJMenuItem1() {
		if (jMenuItem1 == null) {
			jMenuItem1 = new JMenuItem();
			jMenuItem1.setText("Exit");
			jMenuItem1.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
					System.exit(0);
				}
			});
		}
		return jMenuItem1;
	}

	/**
	 * closes the application with a message
	 */
	public void stopIt(String exceptionString) {
		this.getContentPane().remove(canvas1);
		this.getContentPane().repaint();
		canvas1 = null;
		JOptionPane.showMessageDialog(this,
		// "Your system eventually does not support OpenGL 2.0 or the Shader Model 3.0. Please update your graphic driver or your graphic hardware.\n"
		// +
				"The following error was reported:\n" + exceptionString);
		System.exit(0);
	}

}
